package cn.iocoder.yudao.module.order.service.order;

import java.util.*;
import javax.validation.*;

import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.framework.mybatis.core.service.IService;
import cn.iocoder.yudao.module.order.vo.orderTime.*;
import cn.iocoder.yudao.module.order.dal.dataobject.orderTime.OrderTimeDO;
import cn.iocoder.yudao.framework.common.pojo.PageResult;

/**
 * 下单时间轴事件 Service 接口
 *
 * @author 系统管理员
 */
public interface OrderTimeService extends IService<OrderTimeDO> {

    /**
     * 创建下单时间轴事件
     *
     * @param createReqVO 创建信息
     * @return 编号
     */
    Long createOrderTime(@Valid OrderTimeCreateReqVO createReqVO);

    /**
     * 更新下单时间轴事件
     *
     * @param updateReqVO 更新信息
     */
    void updateOrderTime(@Valid OrderTimeUpdateReqVO updateReqVO);


    /**
     * 更新预计开船时间
     *
     * @param containerNumber 自编号
     * @param transportId 运输方式
     * @param estTime         预计开船时间
     */
    void updateOrderEstBookSeaTime(String containerNumber, Integer transportId, Date estTime);



    /**
     * 更新预计到港时间
     *
     * @param containerNumber 自编号
     * @param transportId 运输方式
     * @param estTime         预计到港时间
     */
    void updateOrderEstShippingTime(String containerNumber, Integer transportId, Date estTime);


    /**
     * 删除下单时间轴事件
     *
     * @param id 编号
     */
    void deleteOrderTime(Long id);


    /**
     * 预装反审回退动态到已入仓
     *
     * @param containerNumber 自编号
     */
    void preReverseReviewDeleteOrderTime(String containerNumber);
    void preReverseReviewDeleteOrderTime(Long orderId);

    /**
     * 分拣反审回退动态到打包
     *
     * @param containerNumber 自编号
     */
    void sortingReverseReviewDeleteOrderTime(String containerNumber);
    void sortingReverseReviewDeleteOrderTime(Long orderId);

    /**
     * 获得下单时间轴事件
     *
     * @param id 编号
     * @return 下单时间轴事件
     */
    OrderTimeDO getOrderTime(Long id);

    /**
     * 获得下单时间轴事件列表
     *
     * @param ids 编号
     * @return 下单时间轴事件列表
     */
    List<OrderTimeDO> getOrderTimeList(Collection<Long> ids);


    /**
     * 获得下单时间轴事件列表
     *
     * @param orderId 订单ID
     * @return 下单时间轴事件列表
     */
    List<OrderTimeDO> getOrderTimeList(Long orderId);

    /**
     * 获得下单时间轴事件分页
     *
     * @param page  分页查询
     * @param query 查询
     * @return 下单时间轴事件分页
     */
    PageResult<OrderTimeDO> getOrderTimePage(OrderTimeQueryVO query, PageVO page);

    /**
     * 获得下单时间轴事件列表, 用于 Excel 导出
     *
     * @param query 查询
     * @return 下单时间轴事件列表
     */
    List<OrderTimeDO> getOrderTimeList(OrderTimeQueryVO query);

    /**
     * @param orderId 订单id
     * @param orderNo 订单编号/提单编号
     * @return
     */
    List<OrderTimeBackVO> orderTimeList(Long orderId, String orderNo);
}
